#ifndef adlpuhu_m_driver_h
#define adlpuhu_m_driver_h

#include "sizes.h"

typedef struct musicdriver
{
	const char *name;
	const char *longdescr;
	int active;
	int maxch;
	
	/* Key used for finding. Also useful for command line option char. */
	int optchar;
	
	/* Return 0 if ok, -1 if not */
	int (*detect)(void);
	void (*reset)(void);
	void (*close)(void);
	
	/* Volumes are in 0..63 range.      */
	/* Note is in midi format: no gaps. */
	void (*noteon)(int ch, int note, int volume);
	void (*noteoff)(int ch);
	void (*volumechange)(int ch, int volume);
	
	void (*patch)(int ch,
				  int noteadd, int midipatch,
				  int c4spd, BYTE *adlibpatch);
				  
	void (*setpan)(int ch, int value); /* value is -64..+64 (left..right) */
	
	void (*midictrl)(int ch, BYTE ctrl, BYTE value);
	void (*bend)(int ch, int value); /* value is -128..+127 (0=0, 64=+1 seminote) */
	
	void (*delay)(unsigned us);
	
	struct musicdriver *next;
} musicdriver;

extern void registerdrivers(musicdriver *driver, ...);
extern musicdriver *finddriver(int optchar);
extern void foreachdriver(void (*rut)(const musicdriver *, void *), void *a);

#endif
